
//
// This source file is part of appleseed.
// Visit https://appleseedhq.net/ for additional information and resources.
//
// This software is released under the MIT license.
//
// Copyright (c) 2010-2013 Francois Beaune, Jupiter Jazz Limited
// Copyright (c) 2014-2018 Francois Beaune, The appleseedhq Organization
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#pragma once

// appleseed.renderer headers.
#include "renderer/modeling/entity/entity.h"

// appleseed.foundation headers.
#include "foundation/image/colorspace.h"
#include "foundation/utility/uid.h"

// appleseed.main headers.
#include "main/dllsymbol.h"

// Standard headers.
#include <cstddef>

// Forward declarations.
namespace foundation    { class CanvasProperties; }
namespace foundation    { class Tile; }
namespace renderer      { class ParamArray; }
namespace renderer      { class Source; }
namespace renderer      { class TextureInstance; }

namespace renderer
{

//
// The base class for textures.
//

class APPLESEED_DLLSYMBOL Texture
  : public Entity
{
  public:
    // Return the unique ID of this class of entities.
    static foundation::UniqueID get_class_uid();

    // Constructor.
    Texture(
        const char*                 name,
        const ParamArray&           params);

    // Return a string identifying the model of this entity.
    virtual const char* get_model() const = 0;

    // Return the color space of the texture.
    virtual foundation::ColorSpace get_color_space() const = 0;

    // Access canvas properties.
    virtual const foundation::CanvasProperties& properties() = 0;

    // Create and return a new source that allows to bind this texture to an entity's input.
    virtual Source* create_source(
        const foundation::UniqueID  assembly_uid,
        const TextureInstance&      texture_instance) = 0;

    // Load a given tile.
    // The tile remains owned by the texture.
    virtual foundation::Tile* load_tile(
        const size_t                tile_x,
        const size_t                tile_y) = 0;

    // Unload a given tile.
    virtual void unload_tile(
        const size_t                tile_x,
        const size_t                tile_y,
        const foundation::Tile*     tile) = 0;
};

}   // namespace renderer
